import os
import sys

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 16
project_path = file_path[0:end]
sys.path.append(project_path)
import mns_common.constant.db_name_constant as db_name_constant
import mns_common.component.common_service_fun_api as common_service_fun_api
import mns_common.component.cache.cache_service as cache_service
import mns_common.utils.cmd_util as cmd_util
import mns_common.utils.data_frame_util as data_frame_util
from loguru import logger
from datetime import datetime
import mns_common.utils.date_handle_util as date_handle_util
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.component.trade_date.trade_date_common_service_api as trade_date_common_service_api

mongodb_util = MongodbUtil('27017')

MAX_NUMBER_KEY = 'max_number_key'
# 实时行情同步任务 python名称
REAL_TIME_SCHEDULER_NAME = "sync_realtime_quotes_task"
# 实时行情补偿任务 python名称
REAL_TIME_TASK_NAME = "realtime_quotes_now_sync"
#
REAL_TIME_TASK_NAME_PATH = 'H:\\real_time_task.bat'
# 定时任务
REAL_TIME_SCHEDULER_NAME_PATH = 'H:\\real_time_scheduler.bat'


def get_real_time_max_number():
    number = common_service_fun_api.realtime_quotes_now_max_number(db_name_constant.REAL_TIME_QUOTES_NOW,
                                                                   'number')

    return number


# 检查数据同步最大值
def check_max_number():
    now_max_number = get_real_time_max_number()
    last_minute_number = cache_service.get_cache(MAX_NUMBER_KEY)
    cache_service.set_cache(MAX_NUMBER_KEY, now_max_number)
    if last_minute_number is None:
        return True
    elif now_max_number == last_minute_number:
        return False
    else:
        return True


#  实时行情数据同步状态check
def run_check_real_time_data_sync_status():
    now_date = datetime.now()
    hour = datetime.hour
    minute = datetime.minute
    str_day = now_date.strftime('%Y-%m-%d')

    if bool(1 - trade_date_common_service_api.is_trade_day(str_day)):
        return False

    if hour == 11 and minute == 31:
        all_cmd_processes = cmd_util.get_all_process()
        if data_frame_util.is_empty(all_cmd_processes):
            return False
        all_cmd_processes_real_time_task = get_real_time_quotes_task(all_cmd_processes)
        if data_frame_util.is_empty(all_cmd_processes_real_time_task):
            return False
        for match_task_one in all_cmd_processes_real_time_task.itertuples():
            try:
                processes_pid = match_task_one.process_pid
                # 关闭当前进程
                cmd_util.kill_process_by_pid(processes_pid)
            except BaseException as e:
                logger.error("关闭实时行情任务异常:{}", e)

    if hour == 12 and minute == 59:
        all_cmd_processes = cmd_util.get_all_process()
        if data_frame_util.is_empty(all_cmd_processes):
            return False
        all_cmd_processes_real_time_task = get_real_time_quotes_task(all_cmd_processes)
        if data_frame_util.is_empty(all_cmd_processes_real_time_task):
            return False
        for match_task_one in all_cmd_processes_real_time_task.itertuples():
            try:
                processes_pid = match_task_one.process_pid
                # 关闭当前进程
                cmd_util.kill_process_by_pid(processes_pid)
            except BaseException as e:
                logger.error("关闭实时行情任务异常:{}", e)
            # 重开定时任务
        cmd_util.open_bat_file(REAL_TIME_SCHEDULER_NAME_PATH)

    if bool(1 - date_handle_util.is_trade_time(now_date)):
        return False
    flag = check_max_number()
    if bool(1 - flag):
        all_cmd_processes = cmd_util.get_all_process()
        if data_frame_util.is_empty(all_cmd_processes):
            return False
        all_cmd_processes_real_time_task = get_real_time_quotes_task(all_cmd_processes)
        if data_frame_util.is_empty(all_cmd_processes_real_time_task):
            return None
        for match_task_one in all_cmd_processes_real_time_task.itertuples():
            try:
                processes_pid = match_task_one.process_pid
                # 关闭当前进程
                cmd_util.kill_process_by_pid(processes_pid)
                # 清空临时数据表
                mongodb_util.remove_all_data(db_name_constant.REAL_TIME_QUOTES_NOW)

            except BaseException as e:
                logger.error("关闭实时行情任务异常:{}", e)
        # 重开任务进程
        cmd_util.open_bat_file(REAL_TIME_TASK_NAME_PATH)


def get_real_time_quotes_task(all_cmd_processes):
    return all_cmd_processes[
        (all_cmd_processes['total_info'].str.contains(REAL_TIME_SCHEDULER_NAME, case=False, na=False))
        | (all_cmd_processes['total_info'].str.contains(REAL_TIME_TASK_NAME, case=False, na=False))]


if __name__ == '__main__':
    while True:
        run_check_real_time_data_sync_status()



import os
import sys

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)

from loguru import logger
import mns_common.api.em.east_money_stock_api as east_money_stock_api
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.utils.date_handle_util as date_handle_util
import mns_common.component.common_service_fun_api as common_service_fun_api
import mns_common.api.akshare.k_line_api as k_line_api
import mns_scheduler.k_line.clean.daily.daily_k_line_clean_common_service as daily_k_line_clean_common_service

mongodb_util = MongodbUtil('27017')


# 多线程接口容易拉爆

def save_one_symbol(symbol, period, end_date, hq, hq_col, real_time_quotes_now):
    stock_hfq_df = k_line_api.stock_zh_a_hist(symbol=symbol, period=period,
                                              start_date=date_handle_util.no_slash_date('1990-12-19'),
                                              end_date=date_handle_util.no_slash_date(end_date),
                                              adjust=hq)
    real_time_quotes_now_one = real_time_quotes_now.loc[real_time_quotes_now['symbol'] == symbol]
    stock_hfq_df.rename(columns={"日期": "date", "开盘": "open",
                                 "收盘": "close", "最高": "high",
                                 "最低": "low", "成交量": "volume",
                                 "成交额": "amount", "振幅": "pct_chg",
                                 "涨跌幅": "chg", "涨跌额": "change",
                                 "换手率": "exchange"}, inplace=True)
    classification = common_service_fun_api.classify_symbol_one(symbol)
    stock_hfq_df['symbol'] = symbol
    stock_hfq_df['name'] = list(real_time_quotes_now_one['name'])[0]
    stock_hfq_df['industry'] = list(real_time_quotes_now_one['industry'])[0]
    stock_hfq_df['_id'] = stock_hfq_df['symbol'] + '-' + stock_hfq_df['date']
    stock_hfq_df['last_price'] = round(((stock_hfq_df['close']) / (1 + stock_hfq_df['chg'] / 100)), 2)
    stock_hfq_df['max_chg'] = round(
        ((stock_hfq_df['high'] - stock_hfq_df['last_price']) / stock_hfq_df['last_price']) * 100, 2)
    stock_hfq_df['amount_level'] = round((stock_hfq_df['amount'] / common_service_fun_api.HUNDRED_MILLION), 2)
    stock_hfq_df['flow_mv'] = round(stock_hfq_df['amount'] * 100 / stock_hfq_df['exchange'], 2)
    stock_hfq_df['flow_mv_sp'] = round(stock_hfq_df['flow_mv'] / common_service_fun_api.HUNDRED_MILLION, 2)
    stock_hfq_df['classification'] = classification
    stock_hfq_df = stock_hfq_df.sort_values(by=['date'], ascending=False)
    stock_hfq_df = daily_k_line_clean_common_service.calculate_exchange_and_k_line_avg_param(stock_hfq_df)
    insert_data(stock_hfq_df, hq_col, symbol)
    logger.info(period + 'k线同步-' + hq + '-' + symbol)
    return stock_hfq_df


def insert_data(stock_hfq_df, hq_col, symbol):
    query = {'symbol': symbol}
    tag = mongodb_util.remove_data(query, hq_col)
    success = tag.acknowledged
    if success:
        mongodb_util.insert_mongo(stock_hfq_df, hq_col)


# k线同步 全量
def sync_all_daily_data(period='daily',
                        hq='hfq',
                        hq_col='stock_hfq_daily',
                        end_date='22220101',
                        symbol_list=None):
    create_db_index(hq_col)

    real_time_quotes_now_es = east_money_stock_api.get_real_time_quotes_all_stocks()
    # exclude b symbol
    real_time_quotes_now = common_service_fun_api.exclude_b_symbol(real_time_quotes_now_es.copy())

    real_time_quotes_now = common_service_fun_api.total_mv_classification(real_time_quotes_now.copy())
    if symbol_list is not None:
        real_time_quotes_now = real_time_quotes_now.loc[real_time_quotes_now['symbol'].isin(symbol_list)]

    real_time_quotes_now = real_time_quotes_now.sort_values(by=['chg'], ascending=False)
    fail_symbol_list = []
    for company_info in real_time_quotes_now.itertuples():
        symbol = company_info.symbol
        try:
            k_line_df = save_one_symbol(symbol, period, end_date, hq, hq_col, real_time_quotes_now)
        except BaseException as e:
            fail_symbol_list.append(symbol)
            logger.error('发生异常:{},{}', symbol, e)

    if len(fail_symbol_list) > 0:
        for symbol in fail_symbol_list:
            try:
                k_line_df = save_one_symbol(symbol, period, end_date, hq, hq_col, real_time_quotes_now)
            except BaseException as e:
                logger.error('发生异常:{},{}', symbol, e)
    return k_line_df


def create_db_index(db_name):
    try:

        mongodb_util.create_index(db_name, [("symbol", 1)])
        mongodb_util.create_index(db_name, [("date", 1)])
        mongodb_util.create_index(db_name, [("symbol", 1), ("date", 1)])
        logger.info("创建索引成功:{}", db_name)
    except BaseException as e:
        logger.warning("创建索引异常:{}", e)


if __name__ == '__main__':
    sync_all_daily_data('daily', 'qfq', 'stock_qfq_daily', None, None)
    sync_all_daily_data('daily', 'qfq', 'stock_qfq_weekly', None, None)
    # sync_all_daily_data('weekly', 'qfq', 'stock_qfq_weekly', None, None)
    # sync_all_daily_data('monthly', 'qfq', 'stock_qfq_monthly', None, None)

    # sync_all_daily_data('monthly', '1990-12-19', 'qfq', 'stock_qfq_monthly', None, None)
    # sync_all_daily_data('daily', '1990-12-19', 'qfq', 'stock_qfq_daily', '2023-10-31', None)
    # sync_all_daily_data('weekly', '1990-12-19', 'qfq', 'stock_qfq_weekly', '2023-10-15', None)
    # sync_all_daily_data('monthly', '1990-12-19', 'qfq', 'stock_qfq_monthly', '2023-09-30', None)



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)

from datetime import datetime
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.utils.date_handle_util as date_handle_util
import mns_common.utils.data_frame_util as data_frame_util
from loguru import logger
import mns_common.component.trade_date.trade_date_common_service_api as trade_date_common_service_api
import pandas as pd
import time

mongodb_util = MongodbUtil('27017')


def update_five_connected_boards_task(stock_zt_pool):
    stock_zt_pool_5_list = stock_zt_pool.loc[stock_zt_pool['connected_boards_numbers'] == 5]
    if data_frame_util.is_empty(stock_zt_pool_5_list):
        return None
    # 保存五板以上数据
    handle_five_connected_boards_symbol(stock_zt_pool_5_list)
    time.sleep(10)
    update_high_point_day()


# 连续五板涨停信息同步
def handle_five_connected_boards_symbol(stock_zt_pool_5_list):
    trade_date_list = mongodb_util.find_all_data('trade_date_list')
    str_day_list = list(trade_date_list['trade_date'])
    stock_zt_pool_5_list = stock_zt_pool_5_list.loc[stock_zt_pool_5_list['str_day'].isin(str_day_list)]
    for stock_one in stock_zt_pool_5_list.itertuples():
        try:
            str_day = stock_one.str_day
            # 上市时间
            list_date = str(stock_one.list_date)
            # 五板的时间
            date_str_day = date_handle_util.str_to_date(str_day, "%Y-%m-%d")

            list_date_time = date_handle_util.add_date_day(list_date[0:8], 0)

            delta = date_str_day - list_date_time
            days = delta.days
            new_stock = False
            if days <= 30:
                new_stock = True

            before_five_day = trade_date_common_service_api.get_before_trade_date(str_day, 5)
            stock_zt_pool_five = mongodb_util.find_query_data('stock_zt_pool', query={'symbol': stock_one.symbol,
                                                                                      'str_day': before_five_day})
            stock_zt_pool_five['new_stock'] = new_stock

            save_zt_data(stock_zt_pool_five, stock_one.symbol, str_day)
        except BaseException as e:
            logger.error("发生异常:{},{}", stock_one.symbol, str_day)


def save_zt_data(stock_zt_pool_five, symbol, str_day):
    try:
        stock_zt_pool_five['closure_funds_level'] = round((stock_zt_pool_five['closure_funds'] / 10000000), 2)
        stock_zt_pool_five['amount_level'] = round((stock_zt_pool_five['amount'] / 10000000), 2)

        stock_zt_pool_five = stock_zt_pool_five.rename(columns={
            "str_day": "before_five_day"})

        stock_zt_pool_five['five_boards_day'] = str_day

        high_point_day = calculate_high_point_day(symbol, str_day)

        stock_zt_pool_five.loc[:, "high_point_day"] = high_point_day

        stock_zt_pool_five = stock_zt_pool_five[[
            "_id",
            'symbol',
            "name",
            "now_price",
            "exchange",
            "five_boards_day",
            "before_five_day",
            "high_point_day",
            "industry",
            "flow_mv_level",
            "amount_level",
            "closure_funds_level",
            "flow_mv_sp",
            "total_mv_sp",
            "amount",
            "chg",
            "frying_plates_numbers",
            "statistics",
            "flow_mv",
            "total_mv",
            "closure_funds",
            "first_closure_time",
            "last_closure_time",
            "index",
            'new_stock',
            "connected_boards_numbers",
            "first_sw_industry",
            "second_sw_industry",
            "third_sw_industry",
            "mv_circulation_ratio",
            "list_date",
            "classification",
        ]]

        mongodb_util.save_mongo(stock_zt_pool_five, 'stock_zt_pool_five')
    except BaseException as e:
        logger.error("发生异常:{}", symbol + '-' + str_day)


# 计算最大连板数
def calculate_max_connected_boards_numbers(begin_day, end_day, symbol):
    query = {'symbol': symbol,
             '$and': [{'str_day': {'$gte': begin_day}}, {'str_day': {'$lte': end_day}}]}
    stock_zt_pool = mongodb_util.descend_query(query, 'stock_zt_pool', 'connected_boards_numbers', 1)
    if stock_zt_pool.shape[0] > 0:
        max_connected_boards_numbers = list(stock_zt_pool.connected_boards_numbers)[0]
    else:
        max_connected_boards_numbers = 0
    return max_connected_boards_numbers


# 计算涨停数目
def calculate_total_zt_num(begin_day, end_day, symbol):
    query = {'symbol': symbol,
             '$and': [{'str_day': {'$gte': begin_day}}, {'str_day': {'$lte': end_day}}]}
    return mongodb_util.count(query, 'stock_zt_pool')


# 计算最高点到第一板的最大涨幅
def calculate_max_chg(begin_day, end_day, symbol):
    query_max = {'symbol': symbol, "date": date_handle_util.no_slash_date(end_day)}
    max_stock_hfq_daily = mongodb_util.find_one_query('stock_qfq_daily', query_max)
    query_min = {'symbol': symbol, "date": {'$lte': date_handle_util.no_slash_date(begin_day)}}
    min_stock_hfq_daily = mongodb_util.descend_query(query_min, 'stock_qfq_daily', 'date', 1)
    if max_stock_hfq_daily.shape[0] > 0 and min_stock_hfq_daily.shape[0] > 0:
        max_price = max_stock_hfq_daily['high']
        min_price = min_stock_hfq_daily['last_price']
        max_chg = round(((max_price - min_price) / min_price) * 100, 2)
        max_chg = list(max_chg)[0]
    else:
        max_chg = 0
    min_price = list(min_price)[0]
    start_df = pd.DataFrame([[max_chg, min_price]], columns=['max_chg',
                                                             'start_price'])

    return start_df


# 找出最高点时间
def calculate_high_point_day(symbol, five_boards_day):
    query = {'symbol': symbol, 'date': {'$gte': date_handle_util.no_slash_date(five_boards_day)}}
    stock_hfq_daily_list = mongodb_util.ascend_query(query, 'stock_hfq_daily', 'date', 40)

    if stock_hfq_daily_list.shape[0] == 0:
        return five_boards_day

    max_index = stock_hfq_daily_list['high'].idxmax()
    max_row = stock_hfq_daily_list.iloc[max_index, :]

    date = max_row.date

    high_point_day_date = date_handle_util.str_to_date(date, '%Y%m%d')

    high_point_day = high_point_day_date.strftime('%Y-%m-%d')

    return high_point_day


# 计算最高点 (1+10%)的五次方 = 1.62  (1+10%)的六次方 =1.77
def update_high_point_day():
    stock_zt_pool_five_list = mongodb_util.find_all_data('stock_zt_pool_five')

    stock_zt_pool_five_list = stock_zt_pool_five_list.sort_values(by=['five_boards_day'], ascending=False)

    for stock_one in stock_zt_pool_five_list.itertuples():
        try:
            high_point_day = calculate_high_point_day(stock_one.symbol, stock_one.five_boards_day)
            start_df = calculate_max_chg(stock_one.before_five_day, high_point_day, stock_one.symbol)

            max_connected_boards_numbers = calculate_max_connected_boards_numbers(stock_one.before_five_day,
                                                                                  high_point_day, stock_one.symbol)

            int_date = int(stock_one.list_date)  # 转换为整数
            list_date_01 = datetime(year=int_date // 10000, month=(int_date // 100) % 100, day=int_date % 100)

            str_day_date = date_handle_util.str_to_date(stock_one.five_boards_day, '%Y-%m-%d')
            # 计算日期差值 距离现在上市时间
            diff_days = (str_day_date - list_date_01).days

            total_zt_num = calculate_total_zt_num(stock_one.before_five_day, high_point_day, stock_one.symbol)

            query = {"before_five_day": stock_one.before_five_day, "symbol": stock_one.symbol}
            new_values = {"$set": {"high_point_day": high_point_day,
                                   "max_chg": list(start_df['max_chg'])[0],
                                   "start_price": list(start_df['start_price'])[0],
                                   "max_connected_boards_numbers": max_connected_boards_numbers,
                                   "total_zt_num": total_zt_num,
                                   "diff_days": diff_days
                                   }}
            mongodb_util.update_many(query, new_values, 'stock_zt_pool_five')
        except BaseException as e:
            logger.error("更新 high_point_day 异常:{},{},{}", stock_one.symbol, stock_one.five_boards_day, e)


def fix_miss_high_symbol():
    sync_one_symbol('002901', '2022-09-27')
    sync_one_symbol('000716', '2022-08-24')
    sync_one_symbol('002693', '2022-10-13')
    sync_one_symbol('000948', '2022-10-12')
    sync_one_symbol('000638', '2022-10-11')
    sync_one_symbol('002093', '2022-10-10')
    sync_one_symbol('002907', '2022-10-17')
    sync_one_symbol('002528', '2022-10-12')
    sync_one_symbol('003023', '2022-09-21')
    sync_one_symbol('002093', '2022-10-10')
    sync_one_symbol('003005', '2022-09-28')
    sync_one_symbol('002965', '2022-05-27')

    sync_one_symbol('000032', '2022-10-26')
    sync_one_symbol('002045', '2022-08-10')

    sync_one_symbol('003042', '2022-10-31')

    sync_one_symbol('002896', '2022-07-18')

    sync_one_symbol('000759', '2022-09-08')
    sync_one_symbol('002579', '2022-08-05')

    sync_one_symbol('002992', '2022-07-06')

    sync_one_symbol('000582', '2022-04-07')

    sync_one_symbol('002053', '2022-03-07')


def group_industry():
    pipeline = [
        {'$match': {
            "classification": "S"}},
        {'$group': {'_id': "$first_sw_industry", 'count': {'$sum': 1}}}
    ]
    result = mongodb_util.aggregate(pipeline, 'stock_zt_pool_five')

    result = result.sort_values(by=['count'], ascending=True)
    print(result)


def group_flow_mv():
    pipeline = [
        {'$match': {
            "new_stock": False,
            "classification": {"$in": ["S", "H"]}}},
        {'$group': {'_id': "$flow_mv_level", 'count': {'$sum': 1}}}
    ]
    result = mongodb_util.aggregate(pipeline, 'stock_zt_pool_five')

    result = result.sort_values(by=['count'], ascending=False)
    print(result)


def sync_one_symbol(symbol, str_day):
    stock_zt_pool_five = mongodb_util.find_query_data('stock_zt_pool', query={'symbol': symbol,
                                                                              'str_day': str_day})
    stock_zt_pool_five['new_stock'] = False
    after_five_day = trade_date_common_service_api.get_further_trade_date(str_day, 5)
    save_zt_data(stock_zt_pool_five, symbol, after_five_day)
